/*
LinkPoint class. Supports being linked left,right, up and down
*/
class LinkPoint {
  color c;
  float x, y, radius;
  boolean linkLeft = false;
  boolean linkRight = false;
  boolean linkUp = false;
  boolean linkDown = false;
  int offset = 6;
  /*
  set a color based on the yPosition of this point by interpolating between our chosen colors
  */
  void setFill() {
    colorMode(HSB);
    fill(lerpColor(greenColor, redColor, map(y, 0, height, 0, 1))); //green to red
  }
  /*
  draws the dot part of the this LinkPoint
  */
  void draw() {
    noStroke();
    setFill();
    shape(circle, x, y, circle.width*scaleFactor, circle.height*scaleFactor);
    fill(255);
    shape(circleWhiteDot, x, y, circleWhiteDot.width*scaleFactor, circleWhiteDot.height*scaleFactor);
  }
  /*
  Draws the links of this point. It's looks a little daunting but it's only because we have to
  rotate the PShapes
  */
  void drawLinks() {
    noStroke();
    setFill();
    if (linkLeft) {
      //left is easy, just draw it to the left
      fill(lerpColor(greenColor, redColor, map(y, 0, height, 0, 1))); //green to red
      shape(link, x, y, link.width*scaleFactor, link.height*scaleFactor);
      fill(255);
      shape(linkWhite, x, y, linkWhite.width*scaleFactor-offset, linkWhite.height*scaleFactor);
    }
    if (linkUp) {
      //up we have to rotate then place
      pushMatrix();
      translate(x, y);
      rotate(-PI/2.0);
      fill(lerpColor(greenColor, redColor, map(y, 0, height, 0, 1))); //green to red
      shape(link, -47.5*scaleFactor, 0, link.width*scaleFactor, link.height*scaleFactor);
      fill(255);
      shape(linkWhite, -47.5*scaleFactor, 0, linkWhite.width*scaleFactor-offset, linkWhite.height*scaleFactor);
      popMatrix();
    }
    if (linkRight) {
      //right is easy, just draw it on the right
      fill(lerpColor(greenColor, redColor, map(y, 0, height, 0, 1))); //green to red
      shape(link, x-distBetweenPoints, y, link.width*scaleFactor, link.height*scaleFactor);
      fill(255);
      shape(linkWhite, x-distBetweenPoints, y, linkWhite.width*scaleFactor - offset, linkWhite.height*scaleFactor);
    }
    if (linkDown) {
      //down again we have to rotate then place
      pushMatrix();
      translate(x, y);
      rotate(PI/2.0);
      fill(lerpColor(greenColor, redColor, map(y, 0, height, 0, 1))); //green to red
      shape(link, 0-0, 0-47.5*scaleFactor, link.width*scaleFactor - offset-1, link.height*scaleFactor);
      fill(255);
      shape(linkWhite, 0, -47.5*scaleFactor, linkWhite.width*scaleFactor, linkWhite.height*scaleFactor);
      popMatrix();
    }
  }
  /*
  clears the links for the LinkPoint
  */
  void resetLinks() {
    linkLeft = false;
    linkRight = false;
    linkUp = false;
    linkDown = false;
  }
}

