/* @pjs preload="circle.svg, circleWhiteDot.svg, link.svg, linkWhite.svg"; 
 * Created by Pitch Interactive (info@pitchinteractive.com) www.pitchinteractive.com
 * Created on 12/12/2011
 */
float scaleFactor = 0.40; //have a scaleFactor to resize the entire thing as needed

int distBetweenPoints = int(scaleFactor * 53); //53 is chosen based on the .svg assets we are using
int numRows, numCols; 
ArrayList<LinkPoint> points;
float linkDensity = 0.5; //pct of links we are going to draw
//stores are assets, we need a circle and a link asset
PShape circle;
PShape link;
//as well as some white overlays
PShape linkWhite;
PShape circleWhiteDot;
//we are going to interpolate between these two colors
color redColor = color(179, 43, 44);
color greenColor = color(77, 153, 54);
void setup() {
  size(660,890); //not saved to PDF, opens in Processing window 
  smooth();
  noLoop();
  points = new ArrayList<LinkPoint>();
  //load are shapes
  circle = loadShape("circle.svg");
  link = loadShape("link.svg");
  linkWhite = loadShape("linkWhite.svg");
  circleWhiteDot = loadShape("circleWhiteDot.svg");
  //important, disabling the styles
  //this allows us to use processings fill() and stroke() to style them programitcally
  circle.disableStyle();
  link.disableStyle();
  linkWhite.disableStyle();
  circleWhiteDot.disableStyle();
  setupPoints();
  setupLinks();
}
/*
Adds a bunch of points to the ArrayList arranging them in a grid
. . . . 
. . . . 
. . . .
*/
void setupPoints() {
  points.clear();
  numRows = height / distBetweenPoints;
  numCols = width / distBetweenPoints;
  //println(numRows + " " + numCols);
  for (int y = 0; y < numRows; y++) {
    for (int x = 0; x < numCols; x++) {
      float xPos = (x +.25) * distBetweenPoints; //a slight offset to center things on screen
      float yPos = (y +.25) * distBetweenPoints;
      LinkPoint lp = new LinkPoint();
      lp.x = xPos;
      lp.y = yPos;

      points.add(lp);
    }
  }
}
/*
Sets up the links between the points
trying to make it random!
*/
void setupLinks() {
  for (LinkPoint lp : points) { 
    lp.resetLinks(); //ensure no points are linked up. 
  }
  int numLinks = (numRows - 1) * numCols + (numCols - 1) * numRows; //the total number of links, if we linked Everything
  int linksToDraw =round(linkDensity * numLinks); //the actual number of links we want
  for (int i = 0; i < linksToDraw; i++) {
    int randomPoint = floor(random(points.size())); //find a random point
    LinkPoint lp = points.get(randomPoint);
    //if the random point is completely linked up, find a new random point
    //until we find one that isn't completely linked
    while (lp.linkLeft && lp.linkUp && lp.linkRight && lp.linkDown) {
      lp = points.get(floor(random(points.size())));
    }
    //choose a random direction
    int direction = floor(random(4));
    boolean linkMade = false;
    while (!linkMade) {
      switch(direction) {
      case 0:
        if (!lp.linkLeft) {
          lp.linkLeft = true;
          linkMade = true;
        }
        break;
      case 1:
        if (!lp.linkUp) {
          lp.linkUp = true;
          linkMade = true;
        }
        break;
      case 2:
        if (!lp.linkRight) {
          lp.linkRight = true;
          linkMade = true;
        }
        break;
      case 3:
        if (!lp.linkDown) {
          lp.linkDown = true;
          linkMade = true;
        }
        break;
      }
      //if the chosen direction was already linked
      //go on to the next direction
      direction = (direction + 1) % 4;
    }
  }
  //clean up the edges
  removeSideLinks();
  //randomly choose a word to draw, joy or peace
  if(random(1) < 0.5) {
    textJoy();
  } else {
    textPeace();
  }
}
/*
This is a helper method for drawing text in the links
It clears out all the links for a particular area
*/
void clearLinks(int startRow, int startCol, int numColsToClear, int numRowsToClear) {
  for (int i = startRow; i < startRow + numRowsToClear; i++) {
    for (int j = startCol; j < startCol + numColsToClear; j++) {
      LinkPoint lp = points.get(i * numCols + j);
      //if the point is on the edge of our boundry we only need to clear the edge links
      if (i == startRow) {
        lp.linkDown = false;
      } 
      else if (i == startRow + numRowsToClear - 1) {
        lp.linkUp = false;
      } 
      else if (j == startCol) {
        lp.linkLeft = false;
      } 
      else if (j == startCol + numColsToClear -1) {
        lp.linkRight = false;
      } 
      else {
        //if it's not on the edge, clear them all
        lp.resetLinks();
      }
    }
  }
}
/*
Draws the word JOY in link text
*/
void textJoy() {
  int startRow = 7;
  int startCol = 10;
  int numColsToClear = 11;
  int numRowsToClear = 7;
  //first clear out the links for where JOY will be placed
  clearLinks(startRow, startCol, numColsToClear, numRowsToClear);
  //setup links to build letters
  //this was a pain... :)
  //J
  points.get((startRow + 4) * numCols + startCol+1).linkDown = true;
  points.get((startRow + 5) * numCols + startCol+1).linkLeft = true;
  points.get((startRow + 5) * numCols + startCol+2).linkLeft = true;
  points.get((startRow + 4) * numCols + startCol+3).linkDown = true;
  points.get((startRow + 3) * numCols + startCol+3).linkDown = true;
  points.get((startRow + 2) * numCols + startCol+3).linkDown = true;
  points.get((startRow + 1) * numCols + startCol+3).linkDown = true;
  //O
  points.get((startRow + 1) * numCols + startCol+4).linkDown = true;
  points.get((startRow + 1) * numCols + startCol+4).linkLeft = true;
  points.get((startRow + 1) * numCols + startCol+5).linkLeft = true;
  points.get((startRow + 1) * numCols + startCol+6).linkDown = true;
  points.get((startRow + 2) * numCols + startCol+4).linkDown = true;
  points.get((startRow + 2) * numCols + startCol+6).linkDown = true;
  points.get((startRow + 3) * numCols + startCol+4).linkDown = true;
  points.get((startRow + 3) * numCols + startCol+6).linkDown = true;
  points.get((startRow + 4) * numCols + startCol+4).linkDown = true;
  points.get((startRow + 4) * numCols + startCol+6).linkDown = true;
  points.get((startRow + 5) * numCols + startCol+4).linkLeft = true;
  points.get((startRow + 5) * numCols + startCol+5).linkLeft = true;
  //Y
  points.get((startRow + 1) * numCols + startCol + 7).linkDown = true;
  points.get((startRow + 1) * numCols + startCol + 9).linkDown = true;
  points.get((startRow + 2) * numCols + startCol + 7).linkDown = true;
  points.get((startRow + 2) * numCols + startCol + 9).linkDown = true;
  points.get((startRow + 3) * numCols + startCol + 7).linkLeft = true;
  points.get((startRow + 3) * numCols + startCol + 8).linkLeft = true;
  points.get((startRow + 3) * numCols + startCol + 8).linkDown = true;
  points.get((startRow + 4) * numCols + startCol + 8).linkDown = true;
}
/*
Similar to textJoy but draws peace instead. Same idea though.
*/
void textPeace() {
  int startRow = 7;
  int startCol = 7;
  int numColsToClear = 17;
  int numRowsToClear = 7;
  clearLinks(startRow, startCol, numColsToClear, numRowsToClear);
  //P
  points.get((startRow+1) * numCols + startCol+1).linkLeft = true;
  points.get((startRow+1) * numCols + startCol+1).linkDown = true;
  points.get((startRow+2) * numCols + startCol+1).linkDown = true;
  points.get((startRow+3) * numCols + startCol+1).linkLeft = true;
  points.get((startRow+3) * numCols + startCol+2).linkLeft = true;
  points.get((startRow+1) * numCols + startCol+2).linkLeft = true;
  points.get((startRow+1) * numCols + startCol+3).linkDown = true;
  points.get((startRow+2) * numCols + startCol+3).linkDown = true;
  points.get((startRow+3) * numCols + startCol+1).linkDown = true;
  points.get((startRow+4) * numCols + startCol+1).linkDown = true;
  //E
  points.get((startRow+1) * numCols + startCol + 4).linkLeft = true;
  points.get((startRow+1) * numCols + startCol + 4).linkDown = true;
  points.get((startRow+2) * numCols + startCol + 4).linkDown = true;
  points.get((startRow+1) * numCols + startCol + 5).linkLeft = true;
  points.get((startRow+3) * numCols + startCol + 4).linkLeft = true;
  points.get((startRow+3) * numCols + startCol + 4).linkDown = true;
  points.get((startRow+4) * numCols + startCol + 4).linkDown = true;
  points.get((startRow+5) * numCols + startCol + 4).linkLeft = true;
  points.get((startRow+5) * numCols + startCol + 5).linkLeft = true;
  //A
  points.get((startRow + 1) * numCols + startCol + 7).linkLeft = true;
  points.get((startRow + 1) * numCols + startCol + 8).linkLeft = true;
  points.get((startRow + 1) * numCols + startCol + 7).linkDown = true;
  points.get((startRow + 3) * numCols + startCol + 7).linkLeft = true;
  points.get((startRow + 3) * numCols + startCol + 8).linkLeft = true;
  points.get((startRow + 2) * numCols + startCol + 7).linkDown = true;
  points.get((startRow + 3) * numCols + startCol + 7).linkDown = true;
  points.get((startRow + 4) * numCols + startCol + 7).linkDown = true;
  points.get((startRow + 1) * numCols + startCol + 9).linkDown = true;
  points.get((startRow + 2) * numCols + startCol + 9).linkDown = true;
  points.get((startRow + 3) * numCols + startCol + 9).linkDown = true;
  points.get((startRow + 4) * numCols + startCol + 9).linkDown = true;
  //C
  points.get((startRow + 1) * numCols + startCol + 10).linkLeft = true;
  points.get((startRow + 1) * numCols + startCol + 11).linkLeft = true;
  points.get((startRow + 1) * numCols + startCol + 10).linkDown = true;
  points.get((startRow + 2) * numCols + startCol + 10).linkDown = true;
  points.get((startRow + 3) * numCols + startCol + 10).linkDown = true;
  points.get((startRow + 4) * numCols + startCol + 10).linkDown = true;
  points.get((startRow + 5) * numCols + startCol + 10).linkLeft = true;
  points.get((startRow + 5) * numCols + startCol + 11).linkLeft = true;
  //E
  points.get((startRow+1) * numCols + startCol + 13).linkLeft = true;
  points.get((startRow+1) * numCols + startCol + 13).linkDown = true;
  points.get((startRow+2) * numCols + startCol + 13).linkDown = true;
  points.get((startRow+1) * numCols + startCol + 14).linkLeft = true;
  points.get((startRow+3) * numCols + startCol + 13).linkLeft = true;
  points.get((startRow+3) * numCols + startCol + 13).linkDown = true;
  points.get((startRow+4) * numCols + startCol + 13).linkDown = true;
  points.get((startRow+5) * numCols + startCol + 13).linkLeft = true;
  points.get((startRow+5) * numCols + startCol + 14).linkLeft = true;
}
/*
Method to remove any frayed edges at the sides of the card
*/
void removeSideLinks() {
  int bottomRow = (numRows - 1) * numCols;
  int rightCol = numCols - 1;
  //remove the top and bottom edges
  for (int i = 0; i < numCols; i++) {
    points.get(i).linkUp = false;
    points.get(i + bottomRow).linkDown = false;
  }
  //left and right edges
  for (int i = 0; i < numRows; i++) {
    points.get(i * numCols).linkRight = false;
    points.get(i * numCols + rightCol).linkLeft = false;
  }
}
void draw() {
  background(255);
  noStroke();
  //draw points
  for (LinkPoint lp : points) {
    lp.draw();
  }
  //draw links
  for (LinkPoint lp : points) {
    lp.drawLinks();
  }
}


